CCBreaker = AceLibrary("AceAddon-2.0"):new("Parser-3.0","FuBarPlugin-2.0","AceConsole-2.0","AceDB-2.0")
local CCBabble  = AceLibrary("Babble-Spell-2.2")
local CCLocale  = AceLibrary("AceLocale-2.2"):new("CCBreaker")
local CCTablet = AceLibrary("Tablet-2.0")

CCBreaker.revision = tonumber(string.find("$Revision$","%d+")) or 1

-- Database

CCBreaker:RegisterDB("CCBreakerDB","CCBreakerDBPC")

-- basic datastructures

local oddline = {
	['text']="",
	['text2']="",
	['text3']="",
	['text4']="",
	['textR']=1,
	['textG']=1,
	['textB']=1,
	['text2R']=1,
	['text2G']=1,
	['text2B']=1,
	['text3R']=1,
	['text3G']=1,
	['text3B']=1,
	['text4R']=1,
	['text4G']=1,
	['text4B']=1,
	['justify']="LEFT",
	['justify2']="LEFT",
	['justify3']="LEFT",
	['justify4']="LEFT",
}

local evenline = {
	['text']="",
	['text2']="",
	['text3']="",
	['text4']="",
	['textR']=0.7,
	['textG']=0.7,
	['textB']=0.7,
	['text2R']=0.7,
	['text2G']=0.7,
	['text2B']=0.7,
	['text3R']=0.7,
	['text3G']=0.7,
	['text3B']=0.7,
	['text4R']=0.7,
	['text4G']=0.7,
	['text4B']=0.7,
	['justify']="LEFT",
	['justify2']="LEFT",
	['justify3']="LEFT",
	['justify4']="LEFT",
}

-- table recycling


function CCBreaker:recyclebreakertable()
	while #self.breakers > 0 do
		table.insert(self.breakertablecache, table.remove(self.breakers))
	end
end

function CCBreaker:getbreakertable()
	local t
	if #self.breakertablecache > 0 then
		t = table.remove(self.breakertablecache)
	else
		t = {}
	end
	return t
end

CCBreaker.breakers = {}
CCBreaker.breakertablecache = {}

-- Initialisation

CCBreaker.hasIcon = true

function CCBreaker:OnInitialize()
	self.defaultMinimapPosition = 300
end

function CCBreaker:OnEnable()
	self:RegisterParserEvent({eventType = "Dispel"},"broken")
end

function CCBreaker:OnDisable()
end

-- Workload

function CCBreaker:broken(info)
	if CCBreaker.db.char.debug then
		CCBreaker:PrintLiteral(info)
		CCBreaker:PrintLiteral(CCBreaker:filter(info))
		self:PrintLiteral(self.db.char.display.console)
	end
	if CCBreaker:filter(info) then
		self.lastinfo = info
		self:UpdateText()
		self.lastinfo = nil
		self:displayConsole(info)
		self:displayCenter(info)
		
		--tooltip
		local breaker = self:getbreakertable()
		if (info.recipientAbilityName) then 
		breaker.spell = info.recipientAbilityName
		else
			breaker.spell = ""
		end
	
		if (info.recipientName) then 
			breaker.target = info.recipientName
		else
			breaker.target = ""
		end
		
		if (info.sourceAbilityName) then 
			breaker.ability = info.sourceAbilityName
		else
			breaker.ability = ""
		end
		
		if (info.sourceName) then 
			breaker.breaker = info.sourceName
		else
			breaker.breaker = ""
		end
		table.insert(self.breakers,breaker)
	end
end

function CCBreaker:createText(info,textatt)
	local hlcolor
	local basecolor
	local text
	strings = textatt.strings
	if (strings.two =="") then
		strings.two = CCLocale["[spell] on [target] was removed"]
	end
	if (strings.three =="") then
		strings.three = CCLocale["[spell] on [target] was removed by [breaker]"]
	end
	if (strings.four =="") then
		strings.four = CCLocale["[spell] on [target] was removed by [breaker]'s [ability]"]
	end

	if textatt.colors then
		hlcolor = textatt.color.colortext
		basecolor = "|r"
	else 
		hlcolor = ""
		basecolor = ""
	end
	text = basecolor
	if info.sourceName then
	  if info.sourceAbilityName then
		  text = string.gsub(strings.four,"%[ability%]",hlcolor..info.sourceAbilityName..basecolor)
	  else
		  text = strings.three
	  end
	  text = string.gsub(text,"%[breaker%]",hlcolor..info.sourceName..basecolor)
	else
	  text = strings.two
	end
	text = string.gsub(text,"%[spell%]",hlcolor..info.recipientAbilityName..basecolor)
	text = string.gsub(text,"%[target%]",hlcolor..info.recipientName..basecolor)
	return text
end

function CCBreaker:displayConsole(info)
	if self.db.char.display.console.enable then
		self:Print(self:createText(info,self.db.char.display.console))
	end
end

function CCBreaker:displayCenter(info)
	if self.db.char.display.center.enable then
		UIErrorsFrame:AddMessage(self:createText(info,self.db.char.display.center),
			1.0, 1.0, 1.0, 5.0)
	end
end


function CCBreaker:filter(info)
	local sourceid
	local targetid
	if info.sourceID then
		sourceid = string.gsub(info.sourceID,"%d+","")
	else 
		sourceid = nil
	end
	if info.recipientID then
		targetid = string.gsub(info.recipientID,"%d+","")
	else
		targetid = nil
	end
	local spell	= info.recipientAbilityName
	local showon = self.db.char.showon
	local showfrom = self.db.char.showfrom
	local showeffects = self.db.char.showeffects
	return (
		-- "Show on" filter 
		(((targetid == "player") and (showon.player)) or
		((targetid == "party") and (showon.party)) or
		((targetid == "raid") and (showon.raid)) or
		((targetid == "pet") and (showon.pet)) or
		((targetid == "partypet") and (showon.partypet)) or
		((targetid == "raidpet") and (showon.raidpet)) or
		((targetid == nil ) and (showon.others))) 
		and
		-- "Show From" filter
		(((sourceid == "player") and (showfrom.player)) or
		((sourceid == "partyN") and (showfrom.party)) or
		((sourceid == "raidN") and (showfrom.raid)) or
		((sourceid == "pet") and (showfrom.pet)) or
		((sourceid == "partapetN") and (showfrom.partypet)) or
		((sourceid == "raidpetN") and (showfrom.raidpet)) or
		((sourceid == nil) and (showfrom.others)))
		and
		-- "Show Effect" filter
		(((spell == CCBabble["Shackle Undead"]) and (showeffects.shackleundead)) or
		((spell == CCBabble["Hibernate"]) and (showeffects.hibernate)) or
		((spell == CCBabble["Sap"]) and (showeffects.sap)) or
		(((spell == CCBabble["Polymorph"]) or (spell == CCBabble["Polymorph: Pig"]) 
			or (spell == CCBabble["Polymorph: Turtle"])) and (showeffects.polymorph)) or
		((spell == CCBabble["Freezing Trap Effect"]) and (showeffects.freezingtrap)))
	)   
end

function CCBreaker:OnTextUpdate()
	local text = nil
	local info = self.lastinfo
	local fubar = self.db.char.display.fubar
	if ((info == nil) or not
		(fubar.spell or 
		fubar.target or
		fubar.breaker or
		fubar.ability
		))then 
		text = "CCBreaker"
	else
		if fubar.spell then 
			text = info.recipientAbilityName
		end
		if fubar.target then
			if text then text =  text .. "|" else text = "" end
			text = text..info.recipientName
		end
		if (fubar.breaker and info.sourceName) then
			if text then text =  text .. "|" else text = "" end
			text = text..info.sourceName
		end
		if (fubar.ability and info.sourceAbilityName) then
			if text then text =  text .. "|" else text = "" end
			text = text..info.sourceAbilityName
		end
	end 
		self:SetText(text)
end

function CCBreaker:OnTooltipUpdate()
	local cat = CCTablet:AddCategory(
		'text',"",
		'columns', 4
	)
	
	cat:AddLine(
		'text', CCLocale["spell"],
		'text2',CCLocale["target"],
		'text3',CCLocale["breaker"],
		'text4',CCLocale["ability"],
		'textR',0,
		'textG',1,
		'textB',0,
		'text2R',0,
		'text2G',1,
		'text2B',0,
		'text3R',0,
		'text3G',1,
		'text3B',0,
		'text4R',0,
		'text4G',1,
		'text4B',0,
		'justify',"LEFT",
		'justify2',"LEFT",
		'justify3',"LEFT",
		'justify4',"LEFT"
	)

	local isodd = true
	local current
	
	for i=1, #self.breakers do
		if isodd then current = oddline else current = evenline end
		isodd = not isodd
		current['text'] = self.breakers[#self.breakers+1-i].spell
		current['text2'] = self.breakers[#self.breakers+1-i].target
		current['text3'] = self.breakers[#self.breakers+1-i].breaker
		current['text4'] = self.breakers[#self.breakers+1-i].ability
		cat:AddLine(current)
	end
	
	CCTablet:SetHint("Rightclick for options")
end